"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useCreateReducerWithEffect;
exports.SideEffect = exports.UpdateWithSideEffect = exports.NoUpdate = exports.Update = void 0;

var _react = require("react");

const NO_UPDATE_SYMBOL = Symbol("NO_UPDATE_SYMBOL");

const Update = newState => ({
  newState
});

exports.Update = Update;

const NoUpdate = () => NO_UPDATE_SYMBOL;

exports.NoUpdate = NoUpdate;

const UpdateWithSideEffect = (newState, newSideEffect) => ({
  newState,
  newSideEffect
});

exports.UpdateWithSideEffect = UpdateWithSideEffect;

const SideEffect = newSideEffect => ({
  newSideEffect
});

exports.SideEffect = SideEffect;

async function executeSideEffects({
  sideEffects,
  state,
  dispatch
}) {
  if (sideEffects) {
    while (sideEffects.length) {
      const sideEffect = sideEffects.shift();
      sideEffect(state, dispatch);
    }
  }

  return Promise.resolve();
}

function finalReducer(reducer) {
  return function (state, action) {
    if (action === NO_UPDATE_SYMBOL) {
      return state;
    }

    let {
      newState,
      newSideEffect
    } = reducer(state.state, action);
    const newSideEffects = newSideEffect ? [...state.sideEffects, newSideEffect] : state.sideEffects;
    return {
      state: newState || state.state,
      sideEffects: newSideEffects
    };
  };
}

function useCreateReducerWithEffect(reducer, initialState) {
  const [{
    state,
    sideEffects
  }, dispatch] = (0, _react.useReducer)(finalReducer(reducer), {
    state: initialState,
    sideEffects: []
  });
  (0, _react.useEffect)(() => {
    if (sideEffects.length) {
      async function runSideEffects() {
        await executeSideEffects({
          sideEffects,
          state,
          dispatch
        });
      }

      runSideEffects();
    }
  }, [sideEffects]); //eslint-disable-line

  return [state, dispatch];
}